package org.gcube.informationsystem.model.discovery;

import java.util.ArrayList;
import java.util.List;
import java.util.ServiceLoader;

import org.gcube.informationsystem.base.reference.AccessType;
import org.gcube.informationsystem.base.reference.Element;
import org.gcube.informationsystem.base.reference.entities.EntityElement;
import org.gcube.informationsystem.base.reference.properties.PropertyElement;
import org.gcube.informationsystem.base.reference.relations.RelationElement;
import org.gcube.informationsystem.types.TypeBinder;
import org.gcube.informationsystem.types.reference.TypeDefinition;
import org.gcube.informationsystem.types.reference.entities.EntityType;
import org.gcube.informationsystem.types.reference.properties.PropertyDefinition;
import org.gcube.informationsystem.types.reference.properties.PropertyType;
import org.gcube.informationsystem.types.reference.relations.RelationType;
import org.gcube.informationsystem.utils.ISMapper;
import org.gcube.informationsystem.utils.discovery.ISMDiscovery;
import org.gcube.informationsystem.utils.discovery.RegistrationProvider;
import org.gcube.informationsystem.utils.discovery.SchemaAction;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EntityDiscoveryTest {
	
	private static Logger logger = LoggerFactory.getLogger(ISMDiscovery.class);
	
	private void addDiscoveredModel(ISMDiscovery<? extends Element> ismDiscovery) {
		ServiceLoader<? extends RegistrationProvider> regsitrationProviders = ServiceLoader
				.load(RegistrationProvider.class);
		for(RegistrationProvider registrationProvider : regsitrationProviders) {
			List<Package> packages = registrationProvider.getPackagesToRegister();
			for(Package p : packages) {
				ismDiscovery.addPackage(p);
			}
		}
	}
	
	@Test
	public void testDiscovery() throws Exception {
		SchemaAction schemaAction = new SchemaAction() {
			
			@Override
			public <R extends RelationElement<? extends EntityElement,? extends EntityElement>> void manageRelationClass(Class<R> r)
					throws Exception {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public <P extends PropertyElement> void managePropertyClass(Class<P> e) throws Exception {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public <E extends EntityElement> void manageEntityClass(Class<E> e) throws Exception {
				// TODO Auto-generated method stub
				
			}
		};
		List<Package> packages = new ArrayList<Package>();
		
		Class<TypeDefinition> tdClz = TypeDefinition.class;
		packages.add(tdClz.getPackage());
		
		AccessType[] accessTypes = AccessType.values();
		for(AccessType accessType : accessTypes) {
			Class<Element> clz = accessType.getTypeClass();
			packages.add(clz.getPackage());
		}
		
		ServiceLoader<? extends RegistrationProvider> regsitrationProviders = ServiceLoader
				.load(RegistrationProvider.class);
		for(RegistrationProvider registrationProvider : regsitrationProviders) {
			packages.addAll(registrationProvider.getPackagesToRegister());
		}
		ISMDiscovery.manageISM(schemaAction, packages);
	}
	
	@Test
	public void testPropertyDiscovery() throws Exception {
		ISMDiscovery<PropertyElement> propertyDiscovery = new ISMDiscovery<PropertyElement>(PropertyElement.class);
		addDiscoveredModel(propertyDiscovery);
		propertyDiscovery.discover();
		
		for(Class<? extends PropertyElement> propertyClass : propertyDiscovery.getDiscovered()) {
			String json = TypeBinder.serializeType(propertyClass);
			logger.info("{}", json);
			TypeDefinition typeDefinition = TypeBinder.deserializeTypeDefinition(json);
			logger.info("{}", ISMapper.marshal(typeDefinition));
		}
	}
	
	@Test
	public void testEntityTypeDefinitionSerialization() throws Exception {
		logger.info("{}", TypeBinder.serializeType(EntityType.class));
	}
	
	@Test
	public void testDefinitionSerialization() throws Exception {
		logger.info("{}", TypeBinder.serializeType(TypeDefinition.class));
		logger.info("{}", TypeBinder.serializeType(EntityType.class));
		logger.info("{}", TypeBinder.serializeType(RelationType.class));
		logger.info("{}", TypeBinder.serializeType(PropertyType.class));
		logger.info("{}", TypeBinder.serializeType(PropertyDefinition.class));
	}
	
	@Test
	public void testEntityDiscovery() throws Exception {
		ISMDiscovery<EntityElement> entityDiscovery = new ISMDiscovery<>(EntityElement.class);
		addDiscoveredModel(entityDiscovery);
		entityDiscovery.discover();
		
		for(Class<? extends EntityElement> entity : entityDiscovery.getDiscovered()) {
			logger.info("{}", TypeBinder.serializeType(entity));
		}
	}
	
	@Test
	public void testRelationDiscovery() throws Exception {
		@SuppressWarnings("rawtypes")
		ISMDiscovery<RelationElement> relationDiscovery = new ISMDiscovery<>(RelationElement.class);
		addDiscoveredModel(relationDiscovery);
		relationDiscovery.discover();
		
		for(@SuppressWarnings("rawtypes") Class<? extends RelationElement> relation : relationDiscovery.getDiscovered()) {
			logger.info("{}", TypeBinder.serializeType(relation));
		}
	}
	
	
	
}
